@@ -207,8 +207,6 @@ template <class T>
 NumpyAnyArray 
 readVolumeImpl(VolumeImportInfo const & info, std::string order = "")
 {
-    typedef UnstridedArrayTag Stride;
-    
     if(order == "")
         order = detail::defaultOrder();
 
@@ -294,7 +292,7 @@ readVolume(const char * filename, python::object import_type, std::string order
 }
 
 template <class T>
-void writeVolume(NumpyArray<3, Multiband<T> > const & volume,
+void writeVolume(NumpyArray<3, T > const & volume,
                     const char * filename_base, 
                     const char * filename_ext, 
                     python::object export_type,  
@@ -384,8 +382,11 @@ void defineImpexFunctions()
         "\n"
         "   readVolume(filename, dtype='FLOAT', order='') -> Volume\n"
         "\n"
-        "If the volume is stored in a by-slice manner (e.g. one image per\n"
-        "slice), the 'filename' can refer to an arbitrary image from the set.\n"
+        "If the filename refers to a multi-page TIFF file, the images in the file are\n"
+        "interpreted as the z-slices of the volume.\n"
+        "\n"
+        "If the volume is stored in a by-slice manner (e.g. one file per\n"
+        "z-slice), the 'filename' can refer to an arbitrary image from the set.\n"
         "readVolume() then assumes that the slices are enumerated like::\n"
         "\n"
         "   name_base+[0-9]+name_ext\n"
@@ -420,39 +421,45 @@ void defineImpexFunctions()
         "\n"
         "For details see the help for :func:`readImage`.\n");
         
-    multidef("writeVolume", pywriteVolume<Int8, UInt64, Int64, UInt16, Int16, UInt32, Int32, double, float, UInt8>(), 
+    multidef("writeVolume", pywriteVolume<Singleband<Int8>, Singleband<UInt64>, Singleband<Int64>, Singleband<UInt16>, 
+                                          Singleband<Int16>, Singleband<UInt32>, Singleband<Int32>, Singleband<double>, 
+                                          Singleband<float>, Singleband<UInt8>, TinyVector<float, 3>, TinyVector<UInt8, 3> >(), 
        (arg("volume"), arg("filename_base"), arg("filename_ext"), arg("dtype") = "", arg("compression") = ""),
-       "Wrtie a volume as a sequence of images::\n\n"
-       "   writeVolume(volume, filename_base, filename_ext, dtype = '', compression = '')\n\n"
+       "Write a volume as a sequence of images::\n\n"
+       "   writeVolume(volume, filename_base, filename_ext, dtype='', compression='')\n\n"
        "The resulting image sequence will be enumerated in the form::\n\n"
        "    filename_base+[0-9]+filename_ext\n\n"
+       "Write a volume as a multi-page tiff (filename_ext must be an empty string)::\n\n"
+       "   writeVolume(volume, filename, '', dtype='', compression='')\n\n"
        "Parameters 'dtype' and 'compression' will be handled as in :func:`writeImage`.\n\n");
     
     def("readImage", &readImage, 
         (arg("filename"), arg("dtype") = "FLOAT", arg("index") = 0, arg("order") = ""),
         "Read an image from a file::\n"
         "\n"
-        "   readImage(filename, dtype = 'FLOAT', index = 0, order='') -> Image\n"
+        "   readImage(filename, dtype='FLOAT', index=0, order='') -> Image\n"
         "\n"
-        "When import_type is 'UINT8', 'INT16', 'UINT16', 'INT32', 'UINT32',\n"
+        "When 'dtype' is 'UINT8', 'INT16', 'UINT16', 'INT32', 'UINT32',\n"
         "'FLOAT', 'DOUBLE', or one of the corresponding numpy dtypes (numpy.uint8\n"
         "etc.), the returned image will have the requested pixel type. If\n"
         "dtype is 'NATIVE' or '' (empty string), the image is imported with\n"
-        "the original type of the data in the file. Caution: If the requested \n"
+        "the original type of the data in the file. By default, image data are\n"
+        "returned as 'FLOAT' (i.e. numpy.float32). Caution: If the requested \n"
         "dtype is smaller than the original type in the file, values will be\n"
         "clipped at the bounds of the representable range, which may not be the\n"
         "desired behavior.\n"
         "\n"
         "Individual images of sequential formats such as multi-image TIFF can be \n"
-        "accessed via index. The number of images in a file can be checked with the \n"
-        "function :func:`numberImages`.\n"
+        "accessed via 'index'. The number of images in a file can be checked with the \n"
+        "function :func:`numberImages`. Alternatively, :func:`readVolume` can read \n"
+        "an entire multi-page TIFF in one go.\n"
         "\n"
-        "The order parameter determines the axis ordering of the resulting array\n"
+        "The 'order' parameter determines the axis ordering of the resulting array\n"
         "(allowed values: 'C', 'F', 'V'). When order == '' (the default), \n" 
         "'vigra.VigraArray.defaultOrder' is used.\n"
         "\n"
         "Supported file formats are listed by the function :func:`listFormats`.\n"
-        "When filename does not refer to a recognized image file format, an\n"
+        "When 'filename' does not refer to a recognized image file format, an\n"
         "exception is raised. The file can be checked beforehand with the function\n"
         ":func:`isImage`.\n");
         
@@ -460,7 +467,7 @@ void defineImpexFunctions()
        (arg("image"), arg("filename"), arg("dtype") = "", arg("compression") = "", arg("mode") = "w"),
         "Save an image to a file::\n"
         "\n"
-        "   writeImage(image, filename, dtype = '', compression = '', mode = 'w')\n"
+        "   writeImage(image, filename, dtype='', compression='', mode='w')\n"
         "\n"
         "Parameters:\n\n"
         " image:\n"
@@ -503,7 +510,7 @@ void defineImpexFunctions()
         "        create a new file (default)\n"
         "     'a':\n"
         "        append an image to a file or create a new one if the file does \n"
-        "        not exist (only supported by TIFF)\n\n"
+        "        not exist (only supported by TIFF to create multi-page TIFF files)\n\n"
         "Supported file formats are listed by the function vigra.impexListFormats().\n"
         "The different file formats support the following pixel types:\n\n"
         "   BMP:\n"
