@@ -45,6 +45,10 @@
 #include <vigra/labelvolume.hxx>
 #include <vigra/watersheds3d.hxx>
 #include <vigra/seededregiongrowing3d.hxx>
+#include <vigra/multi_watersheds.hxx>
+#include <vigra/convolution.hxx>
+#include <vigra/multi_convolution.hxx>
+#include <vigra/slic.hxx>
 
 #include <string>
 #include <cmath>
@@ -693,6 +697,113 @@ pythonWatersheds2D(NumpyArray<2, Singleband<PixelType> > image,
 
 VIGRA_PYTHON_MULTITYPE_FUNCTOR(pywatersheds2D, pythonWatersheds2D)
 
+template <unsigned int N, class PixelType >
+python::tuple 
+pythonWatershedsNew(NumpyArray<N, Singleband<PixelType> > image,
+                    int neighborhood = 0,
+                    NumpyArray<N, Singleband<npy_uint32> > seeds = NumpyArray<N, Singleband<npy_uint32> >(),
+                    std::string method = "", 
+                    SRGType srgType = CompleteGrow, 
+                    PixelType max_cost = 0.0, 
+                    NumpyArray<N, Singleband<npy_uint32> > res = NumpyArray<N, Singleband<npy_uint32> >())
+{
+    method = tolower(method);
+    if(method == "" || method == "turbo")
+    {
+        method = "regiongrowing";
+    }
+    
+    std::string description("watershed labeling, neighborhood=");
+    description += asString(neighborhood);
+    
+    res.reshapeIfEmpty(image.taggedShape().setChannelDescription(description), 
+            "watersheds(): Output array has wrong shape.");
+    
+    WatershedOptions options;
+    options.srgType(srgType);
+    
+    if(method == "regiongrowing")
+    {
+        options.regionGrowing();
+    }
+    else if(method == "unionfind")
+    {
+        options.unionFind();
+    }
+    else
+    {
+        vigra_precondition(false, "watersheds(): Unknown watershed method requested.");
+    }
+    
+    if(max_cost > 0.0)
+    {
+        vigra_precondition(method != "unionfind",
+           "watersheds(): UnionFind does not support a cost threshold.");
+        options.stopAtThreshold(max_cost);
+    }
+    
+    if(seeds.hasData())
+    {
+        vigra_precondition(method != "unionfind",
+           "watersheds(): UnionFind does not support seed images.");
+        res = seeds;
+    }
+    else
+    {
+        options.seedOptions(SeedOptions().extendedMinima());
+    }
+    
+    NeighborhoodType n = (neighborhood == 0)
+                             ? DirectNeighborhood
+                             : IndirectNeighborhood;    
+    npy_uint32 maxRegionLabel = 0;
+    {
+        PyAllowThreads _pythread;
+        maxRegionLabel = watershedsMultiArray(image, res, n, options);
+    }
+
+    return python::make_tuple(res, maxRegionLabel);
+}
+
+template <class PixelType >
+python::tuple 
+pythonWatersheds2DNew(NumpyArray<2, Singleband<PixelType> > image,
+                      int neighborhood = 4,
+                      NumpyArray<2, Singleband<npy_uint32> > seeds = NumpyArray<2, Singleband<npy_uint32> >(),
+                      std::string method = "", 
+                      SRGType srgType = CompleteGrow, 
+                      PixelType max_cost = 0.0, 
+                      NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
+{
+    vigra_precondition(neighborhood == 4 || neighborhood == 8,
+           "watersheds2D(): neighborhood must be 4 or 8.");
+    neighborhood = (neighborhood == 4)
+                        ? 0
+                        : 1;
+    return pythonWatershedsNew(image, neighborhood, seeds, method, srgType, max_cost, res);
+}
+
+template <class PixelType >
+python::tuple 
+pythonWatersheds3DNew(NumpyArray<3, Singleband<PixelType> > image,
+                      int neighborhood = 6,
+                      NumpyArray<3, Singleband<npy_uint32> > seeds = NumpyArray<3, Singleband<npy_uint32> >(),
+                      std::string method = "", 
+                      SRGType srgType = CompleteGrow, 
+                      PixelType max_cost = 0.0, 
+                      NumpyArray<3, Singleband<npy_uint32> > res = NumpyArray<3, Singleband<npy_uint32> >())
+{
+    vigra_precondition(neighborhood == 6 || neighborhood == 26,
+           "watersheds3D(): neighborhood must be 6 or 26.");
+    neighborhood = (neighborhood == 6)
+                        ? 0
+                        : 1;
+    return pythonWatershedsNew(image, neighborhood, seeds, method, srgType, max_cost, res);
+}
+
+VIGRA_PYTHON_MULTITYPE_FUNCTOR(pywatersheds2DNew, pythonWatersheds2DNew)
+VIGRA_PYTHON_MULTITYPE_FUNCTOR(pywatersheds3DNew, pythonWatersheds3DNew)
+
 template < class PixelType >
 python::tuple 
 pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
@@ -750,7 +861,7 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
             {
                 extendedLocalMinima3D(srcMultiArrayRange(image), destMultiArray(minima),
                                       (npy_uint32)1, NeighborCode3DSix());
-                labelVolumeWithBackground(srcMultiArrayRange(minima),
+                maxRegionLabel = labelVolumeWithBackground(srcMultiArrayRange(minima),
                                           destMultiArray(seeds), NeighborCode3DSix(),
                                           (npy_uint32)0);
             }
@@ -758,7 +869,7 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
             {
                 extendedLocalMinima3D(srcMultiArrayRange(image), destMultiArray(minima),
                                       (npy_uint32)1, NeighborCode3DTwentySix());
-                labelVolumeWithBackground(srcMultiArrayRange(minima),
+                maxRegionLabel = labelVolumeWithBackground(srcMultiArrayRange(minima),
                                           destMultiArray(seeds), NeighborCode3DTwentySix(),
                                           (npy_uint32)0);
             }
@@ -834,13 +945,77 @@ pythonWatersheds3D(NumpyArray<3, Singleband<PixelType> > image,
 
 VIGRA_PYTHON_MULTITYPE_FUNCTOR(pywatersheds3D, pythonWatersheds3D)
 
+template <unsigned int N, class PixelType >
+python::tuple 
+pythonSlic(NumpyArray<N, PixelType > array,
+           double intensityScaling,
+           unsigned int seedDistance,
+           unsigned int minSize = 0,            // choose minSize automatically 
+           unsigned int iterations = 10, 
+           NumpyArray<N, Singleband<npy_uint32> > res = NumpyArray<N, Singleband<npy_uint32> >())
+{
+    typedef typename detail::ResolveMultiband<PixelType>::type ValueType;
+    typedef typename NormTraits<ValueType>::NormType TmpType;
+    
+    std::string description("Slic superpixels");
+    
+    res.reshapeIfEmpty(array.taggedShape().setChannelDescription(description), 
+            "slicSuperpixels(): Output array has wrong shape.");
+    
+    npy_uint32 maxRegionLabel = 0;
+    {
+        PyAllowThreads _pythread;
+        
+        MultiArray<N, TmpType> gradMag(array.shape());
+        
+        // the original code uses the symmetric difference instead of a Gaussian gradient
+        gaussianGradientMagnitude(array, gradMag, 1.0);
+        // search radius of 1 is also used in the original code
+        generateSlicSeeds(gradMag, res, seedDistance, 1);
+        
+        maxRegionLabel = slicSuperpixels(array, res, intensityScaling, seedDistance,
+                                         SlicOptions().iterations(iterations)
+                                                      .minSize(minSize));
+    }
+
+    return python::make_tuple(res, maxRegionLabel);
+}
+
+template <class PixelType >
+python::tuple 
+pythonSlic2D(NumpyArray<2, PixelType > image,
+             double intensityScaling,
+             unsigned int seedDistance,
+             unsigned int minSize = 0,            // choose minSize automatically 
+             unsigned int iterations = 10, 
+             NumpyArray<2, Singleband<npy_uint32> > res = NumpyArray<2, Singleband<npy_uint32> >())
+{
+    return pythonSlic(image, intensityScaling, seedDistance, minSize, iterations, res);
+}
+
+VIGRA_PYTHON_MULTITYPE_FUNCTOR(pySlic2D, pythonSlic2D)
+
+template <class PixelType >
+python::tuple 
+pythonSlic3D(NumpyArray<3, PixelType > image,
+             double intensityScaling,
+             unsigned int seedDistance,
+             unsigned int minSize = 0,            // choose minSize automatically 
+             unsigned int iterations = 10, 
+             NumpyArray<3, Singleband<npy_uint32> > res = NumpyArray<3, Singleband<npy_uint32> >())
+{
+    return pythonSlic(image, intensityScaling, seedDistance, minSize, iterations, res);
+}
+
+VIGRA_PYTHON_MULTITYPE_FUNCTOR(pySlic3D, pythonSlic3D)
+
 void defineSegmentation()
 {
     using namespace python;
     
     docstring_options doc_options(true, true, false);
 
-    multidef("labelImage", pyLabelImage<npy_uint8, float>(),
+    multidef("labelImage", pyLabelImage<npy_uint8, npy_uint32, float>(),
         (arg("image"), 
         arg("neighborhood") = 4,
         arg("out")=python::object()),
@@ -848,7 +1023,7 @@ void defineSegmentation()
         "the pixel neighborhood to be used and can be 4 (default) or 8.\n\n"
         "For details see labelImage_ in the vigra C++ documentation.\n");
 
-    multidef("labelImageWithBackground", pyLabelImageWithBackground<npy_uint8, float>(),
+    multidef("labelImageWithBackground", pyLabelImageWithBackground<npy_uint8, npy_uint32, float>(),
         (arg("image"), 
         arg("neighborhood") = 4,
         arg("background_value") = 0,
@@ -1046,6 +1221,61 @@ void defineSegmentation()
        arg("max_cost")=0,
        arg("out")=python::object()),
        "Likewise, compute watersheds of a volume.\n");
+
+    multidef("watershedsNew", pywatersheds2DNew< npy_uint8, float >(),
+      (arg("image"), 
+       arg("neighborhood") = 4, 
+       arg("seeds")=python::object(), 
+       arg("method")="",
+       arg("terminate")=CompleteGrow,
+       arg("max_cost")=0,
+       arg("out")=python::object()),
+       "graph-based watershed");
+
+    multidef("watershedsNew", pywatersheds3DNew< npy_uint8, float >(),
+      (arg("image"), 
+       arg("neighborhood") = 6, 
+       arg("seeds")=python::object(), 
+       arg("method")="",
+       arg("terminate")=CompleteGrow,
+       arg("max_cost")=0,
+       arg("out")=python::object()),
+       "graph-based watershed");
+
+    multidef("slicSuperpixels", pySlic2D< TinyVector<float, 3>, Singleband<float> >(),
+      (arg("image"), 
+       arg("intensityScaling"), 
+       arg("seedDistance"), 
+       arg("minSize")=0,
+       arg("iterations")=10,
+       arg("out")=python::object()),
+        "Compute Slic superpixels for a 2D image.\n\n"
+
+        "Parameters:\n\n"
+        " image:\n"
+        "    The 2D-image on which the superpixels will be calculated. Accepts single- and threeband images. \n\n"
+        " intensityScaling:\n"
+        "    Scale (divide) color/intensity difference by this parameter before comparing to spatial distance. \n\n"
+        " seedDistance:\n"
+        "    specify the radius of the window around each seed in which the algorithm looks for potential members of the corresponding superpixel"
+        " thus limiting the superpixel size. The grid spacing for seed placement is determined by this parameter.\n\n"
+        " minSize:\n"
+        "    Minimum size for superpixels. By default the algorithm merges all regions smaller than a quarter of the average superpixel size.\n\n"
+        " iterations:\n"
+        "    Specify number of iterations. The default is 10."
+        " out:\n"
+        "    The label image (with dtype=numpy.uint32) to be filled by the algorithm. "
+        "    It will be allocated by the slicSuperpixels function if not provided)\n\n"
+        "The function returns a Python tuple (labelImage, maxRegionLabel)\n\n");
+
+    multidef("slicSuperpixels", pySlic3D< TinyVector<float, 3>, Singleband<float> >(),
+      (arg("image"), 
+       arg("intensityScaling"), 
+       arg("seedDistance"), 
+       arg("minSize")=0,
+       arg("iterations")=10,
+       arg("out")=python::object()),
+       "Likewise compute Slic superpixels for a 3D volume, either single- or threeband.\n");
 }
 
 void defineEdgedetection();
